use env_logger::{Builder, Env};
use log::{error, LevelFilter};
use std::io::Write;

use crate::{get_config_path, read_config, Config};

pub fn init_logger() {
    Builder::from_env(Env::default().default_filter_or("info"))
        .format(|buf, record| {
            writeln!(
                buf,
                "{} [{}] - {}",
                chrono::Local::now().format("%Y:%m:%d %H:%M:%S"),
                record.level(),
                record.args()
            )
        })
        .filter(None, LevelFilter::Info)
        .init();
}

pub async fn get_config() -> Option<Config> {
    let config_path = match get_config_path().await {
        Ok(path) => path,
        Err(tried_paths) => {
            let extra: String = if !tried_paths.is_empty() {
                let joined = tried_paths
                    .iter()
                    .filter_map(|path| path.to_str())
                    .collect::<Vec<_>>()
                    .join(", ");
                format!(", tried the paths: {}", &joined)
            } else {
                String::with_capacity(0)
            };
            error!("Failed to find any config file{}", &extra);
            return None;
        }
    };

    let read_result = read_config(&config_path).await;
    if let Err(e) = &read_result {
        error!("Error occurred while getting config path: {:?}", e);
    }
    read_result.ok()
}
